""" Code audit tool for python.

:copyright: 2013 by Kirill Klenov.
:license: BSD, see LICENSE for more details.

"""

version_info = 2, 0, 2

__version__ = version = '.'.join(map(str, version_info))
__project__ = __name__
__author__ = "Kirill Klenov <horneds@gmail.com>"
__license__ = "GNU LGPL"
